/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

#uses "fwUnitTestComponentAsserts.ctl"

#uses "test/unImportDevice/unImportDeviceTest_helper.ctl"


#uses "unImportDevice.ctl"
#uses "fwDevice/fwDevice.ctl"

const string SIM_DRIVER_1 = "1";
const string DEVICES_DRIVER_NUMBER = "3";


//-------------------------------------------------------
void test_unImportDevice_existingDevices_setupSuite() {
  fwDevice_initialize(); //needed by unImportDevice_checkFile()

  //Do an initial import during the setup, so devices exist for subsequent imports
  bool importOk = true;
  string sImportFile = "test_unImportDevice/wincc_oa_db_file_SoftFE_Create.txt";
  string sLogContent;

  //Start SIM driver 1 and 3
  unImportTestDevice_helper_startDriver(SIM_DRIVER_1);
  unImportTestDevice_helper_startDriver(DEVICES_DRIVER_NUMBER);

  //Import Devices
  importOk = unImportTestDevice_helper_importDevices(sImportFile, DEVICES_DRIVER_NUMBER, sLogContent);

  //Check that all is ok
  bool bIsWarningPresent = strpos(sLogContent, "WARNING") >= 0;
  bool bIsErrorPresent = strpos(sLogContent, "ERROR") >= 0;
  bool bIsImportSuccess = strpos(sLogContent, "file was successfully imported") >= 0;

  assertEqual(true, importOk, "An error occured while importing " + sImportFile + "\n"+sLogContent);
  assertEqual(false, bIsWarningPresent, "There was a warning when importing " + sImportFile + "\n"+sLogContent);
  assertEqual(false, bIsErrorPresent, "There was an error when importing " + sImportFile + "\n"+sLogContent);
  assertEqual(true, bIsImportSuccess, "Import did not complete (log does not contains successful importation statement) "+sImportFile + "\n"+sLogContent);
}

//-------------------------------------------------------
void test_unImportDevice_checkNoRunning_SIMDrivers() {
  bool checkOk = true;
  string sImportFile = "test_unImportDevice/wincc_oa_db_file_SoftFE_Create.txt";
  string sLogContent;

  //Stop SIM driver 1 and 3
  unImportTestDevice_helper_stopDriver(SIM_DRIVER_1);
  unImportTestDevice_helper_stopDriver(DEVICES_DRIVER_NUMBER);

  //Check Devices
  checkOk = unImportTestDevice_helper_checkDevices(sImportFile, "SOFT_FE", DEVICES_DRIVER_NUMBER, sLogContent);

  //Check that SIM driver 1 is not running
  bool bIsErrorPresent_1 = strpos(sLogContent, "ERROR: Start SIM driver 1 before importing the database.") >= 0;
  //Check that SIM driver 3 is not running
  bool bIsErrorPresent_2 = strpos(sLogContent, "ERROR: Start selected SIM driver before importing the database") >= 0;

  assertEqual(true, bIsErrorPresent_1, "An error indicating that SIM driver 1 is not running was expected, but was not invoked");
  assertEqual(true, bIsErrorPresent_2, "An error indicating that SIM driver 3 is not running was expected, but was not invoked");
}
//-------------------------------------------------------
void test_unImportDevice_checkNoRunning_SIM1() {
  bool checkOk = true;
  string sImportFile = "test_unImportDevice/wincc_oa_db_file_SoftFE_Create.txt";
  string sLogContent;

  //Stop SIM driver 1
  unImportTestDevice_helper_stopDriver(SIM_DRIVER_1);
  //Start SIM driver 3
  unImportTestDevice_helper_startDriver(DEVICES_DRIVER_NUMBER);

  //Check Devices
  checkOk = unImportTestDevice_helper_checkDevices(sImportFile, "SOFT_FE", DEVICES_DRIVER_NUMBER, sLogContent);

  //Check that SIM driver 1 is not running
  bool bIsErrorPresent = strpos(sLogContent, "ERROR: Start SIM driver 1 before importing the database.") >= 0;

  assertEqual(true, bIsErrorPresent, "An error indicating that SIM driver 1 is not running was expected, but was not invoked");
}

//-------------------------------------------------------
void test_unImportDevice_checkNoRunning_SelectedSIM() {
  bool checkOk = true;
  string sImportFile = "test_unImportDevice/wincc_oa_db_file_SoftFE_Create.txt";
  string sLogContent;

  //Start SIM driver 1
  unImportTestDevice_helper_startDriver(SIM_DRIVER_1);
  //Stop SIM driver 3
  unImportTestDevice_helper_stopDriver(DEVICES_DRIVER_NUMBER);

  //Check Devices
  checkOk = unImportTestDevice_helper_checkDevices(sImportFile, "SOFT_FE", DEVICES_DRIVER_NUMBER, sLogContent);

  //Check that SIM driver 3 is not running
  bool bIsErrorPresent = strpos(sLogContent, "ERROR: Start selected SIM driver before importing the database") >= 0;

  assertEqual(true, bIsErrorPresent, "An error indicating that SIM driver 3 is not running was expected, but was not invoked");
}

//-------------------------------------------------------
void test_unImportDevice_checkRunning_SIMDrivers() {
  bool checkOk = true;
  string sImportFile = "test_unImportDevice/wincc_oa_db_file_SoftFE_Create.txt";
  string sLogContent;

  //Start SIM driver 1 and 3
  unImportTestDevice_helper_startDriver(SIM_DRIVER_1);
  unImportTestDevice_helper_startDriver(DEVICES_DRIVER_NUMBER);

  //Check Devices
  checkOk = unImportTestDevice_helper_checkDevices(sImportFile, "SOFT_FE", DEVICES_DRIVER_NUMBER, sLogContent);

  //Check that all is ok
  bool bIsWarningPresent = strpos(sLogContent, "WARNING") >= 0;
  bool bIsErrorPresent = strpos(sLogContent, "ERROR") >= 0;
  bool bIsImportSuccess = strpos(sLogContent, "Press import button to import the database...") >= 0;

  assertEqual(true, checkOk, "An error occured while importing " + sImportFile + "\n"+sLogContent);
  assertEqual(false, bIsWarningPresent, "There was a warning when importing " + sImportFile + "\n"+sLogContent);
  assertEqual(false, bIsErrorPresent, "There was an error when importing " + sImportFile + "\n"+sLogContent);
  assertEqual(true, bIsImportSuccess, "Check did not complete (log does not contains successful check statement) "+sImportFile + "\n"+sLogContent);
}

//-------------------------------------------------------
void test_unImportDevice_importNoRunning_SIMDrivers() {
  bool importOk = true;
  string sImportFile = "test_unImportDevice/wincc_oa_db_file_SoftFE_Create.txt";
  string sLogContent;

  //Stop SIM driver 1 and 3
  unImportTestDevice_helper_stopDriver(SIM_DRIVER_1);
  unImportTestDevice_helper_stopDriver(DEVICES_DRIVER_NUMBER);

  //Import Devices
  importOk = unImportTestDevice_helper_importDevices(sImportFile, DEVICES_DRIVER_NUMBER, sLogContent);

  //Check that SIM driver 3 is not running
  bool bIsErrorPresent = strpos(sLogContent, "ERROR: Simulation driver " + DEVICES_DRIVER_NUMBER + " is not running.") >= 0;

  assertEqual(true, bIsErrorPresent, "An error indicating that SIM driver 3 is not running was expected, but was not invoked");
}
//-------------------------------------------------------
void test_unImportDevice_importNoRunning_SIM1() {
  bool importOk = true;
  string sImportFile = "test_unImportDevice/wincc_oa_db_file_SoftFE_Create.txt";
  string sLogContent;

  //Stop SIM driver 1
  unImportTestDevice_helper_stopDriver(SIM_DRIVER_1);
  //Start SIM driver 3
  unImportTestDevice_helper_startDriver(DEVICES_DRIVER_NUMBER);

  //Import Devices
  importOk = unImportTestDevice_helper_importDevices(sImportFile, DEVICES_DRIVER_NUMBER, sLogContent);

  //Check that SIM driver 1 is not running
  bool bIsErrorPresent = strpos(sLogContent, "ERROR: Simulation driver " + SIM_DRIVER_1 + " is not running.") >= 0;

  assertEqual(true, bIsErrorPresent, "An error indicating that SIM driver 1 is not running was expected, but was not invoked");
}

//-------------------------------------------------------
void test_unImportDevice_importNoRunning_SelectedSIM() {
  bool importOk = true;
  string sImportFile = "test_unImportDevice/wincc_oa_db_file_SoftFE_Create.txt";
  string sLogContent;

  //Start SIM driver 1
  unImportTestDevice_helper_startDriver(SIM_DRIVER_1);
  //Stop SIM driver 3
  unImportTestDevice_helper_stopDriver(DEVICES_DRIVER_NUMBER);

  //Import Devices
  importOk = unImportTestDevice_helper_importDevices(sImportFile, DEVICES_DRIVER_NUMBER, sLogContent);

  //Check that SIM driver 3 is not running
  bool bIsErrorPresent = strpos(sLogContent, "ERROR: Simulation driver " + DEVICES_DRIVER_NUMBER + " is not running.") >= 0;

  assertEqual(true, bIsErrorPresent, "An error indicating that SIM driver 3 is not running was expected, but was not invoked");
}

//-------------------------------------------------------
void test_unImportDevice_importRunning_SIMDrivers() {
  bool importOk = true;
  string sImportFile = "test_unImportDevice/wincc_oa_db_file_SoftFE_Create.txt";
  string sLogContent;

  //Start SIM driver 1 and 3
  unImportTestDevice_helper_startDriver(SIM_DRIVER_1);
  unImportTestDevice_helper_startDriver(DEVICES_DRIVER_NUMBER);

  //Import Devices
  importOk = unImportTestDevice_helper_importDevices(sImportFile, DEVICES_DRIVER_NUMBER, sLogContent);

  //Check that all is ok
  bool bIsWarningPresent = strpos(sLogContent, "WARNING") >= 0;
  bool bIsErrorPresent = strpos(sLogContent, "ERROR") >= 0;
  bool bIsImportSuccess = strpos(sLogContent, "file was successfully imported") >= 0;

  assertEqual(true, importOk, "An error occured while importing " + sImportFile + "\n"+sLogContent);
  assertEqual(false, bIsWarningPresent, "There was a warning when importing " + sImportFile + "\n"+sLogContent);
  assertEqual(false, bIsErrorPresent, "There was an error when importing " + sImportFile + "\n"+sLogContent);
  assertEqual(true, bIsImportSuccess, "Import did not complete (log does not contains successful importation statement) "+sImportFile + "\n"+sLogContent);
}

//------------------------------------------------------
void test_unImportDevice_existingDevices_teardownSuite() {
  //Restart SIM drivers in case they were stopped
  unImportTestDevice_helper_startDriver(SIM_DRIVER_1);
  unImportTestDevice_helper_startDriver(DEVICES_DRIVER_NUMBER);
}
