/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

//-------------------------------------------------------
public bool unImportTestDevice_helper_importDevices(string sFileName, int iDriver, string &sLogContent) {
  dyn_string exceptionInfo;
  string logfile = tmpnam();

  sFileName = getPath(DATA_REL_PATH, sFileName);

  if (dynlen(exceptionInfo) > 0) {
      DebugTN(exceptionInfo);
      exit(1);
  }

  bool importOk = unImportDevice_importFile(sFileName,
                                logfile,
                                iDriver,
                                "RDB-99) EVENT",
                                "RDB-99) EVENT",
                                "RDB-99) EVENT",
                                false);

  bool bReadLog = fileToString(logfile, sLogContent);

  remove(logfile);

  return importOk & bReadLog;
}

//-------------------------------------------------------
public bool unImportTestDevice_helper_checkDevices(string sFileName, string sFrontEndType, int iDriver, string &sLogContent) {
  dyn_string exceptionInfo;
  string logfile = tmpnam();
  bool deleteRequested;

  sFileName = getPath(DATA_REL_PATH, sFileName);

  assertEqual(true, sFileName!="", "Unable to find the import file "+ sFileName);

  if (dynlen(exceptionInfo) > 0) {
      DebugTN(exceptionInfo);
      exit(1);
  }


  bool checkOk = unImportDevice_checkFile(sFileName,
                                logfile,
                                sFrontEndType,
                                iDriver,
                                "RDB-99) EVENT",
                                "RDB-99) EVENT",
                                "RDB-99) EVENT",
                                deleteRequested);

  bool bReadLog = fileToString(logfile, sLogContent);

  remove(logfile);

  DebugTN(__FUNCTION__, "log content", sLogContent);

  return checkOk & bReadLog;
}

//-------------------------------------------------------
public void unImportTestDevice_helper_startDriver(int iDriver){
  if (fwInstallationManager_isDriverRunning(iDriver)){
    return;
  }

  if(iDriver == 1) {
    fwInstallationManager_add("WCCILsim", "always", 30, 2, 30, "");
    fwInstallationManager_command("START", "WCCILsim", "");
  } else {
    fwInstallationManager_add("WCCILsim", "always", 30, 2, 30, "-num "+iDriver);
    fwInstallationManager_command("START", "WCCILsim", "-num "+iDriver);
  }

  fwInstallationManager_waitForState(true, iDriver);
}

//-------------------------------------------------------
public void unImportTestDevice_helper_stopDriver(int iDriver){
  if (!fwInstallationManager_isDriverRunning(iDriver)){
    return;
  }

  if(iDriver == 1) {
    fwInstallationManager_setMode("WCCILsim", "", "manual");
    fwInstallationManager_command("KILL", "WCCILsim", "");
  } else {
    fwInstallationManager_setMode("WCCILsim", "-num "+iDriver, "manual");
    fwInstallationManager_command("KILL", "WCCILsim", "-num "+iDriver);
  }

  fwInstallationManager_waitForState(false, iDriver);
}
